//CCVisu is a tool for co-change visualization.
//This file is part of CCVisu. 

//Copyright (C) 2005  Dirk Beyer

//CCVisu is free software; you can redistribute it and/or
//modify it under the terms of the GNU Lesser General Public License
//as published by the Free Software Foundation; either
//version 2.1 of the License, or (at your option) any later version.

//CCVisu is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
//Lesser General Public License for more details.

//You should have received a copy of the GNU Lesser General Public License
//along with CCVisu; if not, write to the Free Software
//Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

//Please find the GNU Lesser General Public License in file
//../license_lgpl.txt or http://www.gnu.org/licenses/lgpl.txt

//Dirk Beyer    (firstname.lastname@epfl.ch)
//Ecole Polytechnique Fdrale de Lausanne (EPFL), Switzerland


package ccvisu;

import java.awt.Color;
import java.io.PrintWriter;
import java.text.DateFormat;
import java.util.Date;

/*****************************************************************
 * Writer for layouts in VRML format.
 * @version  $Revision: 1.21 $; $Date: 2005/08/22 12:30:56 $
 * @author   Dirk Beyer
 *****************************************************************/
public class WriterDataGraphicsVRML extends WriterDataGraphics {

  private PrintWriter out;
  private float scalePos;

  /**
   * Constructor.
   * @param graph       Graph representation, contains the positions of the vertices.
   * @param out         Output stream writer.
   * @param hideSource  If true, draw only vertices that are not source of an edge.
   *                    In co-change graphs, all change-transaction vertices 
   *                    are source vertices.
   * @param minVert     Diameter of the smallest vertex.
   * @param fontSize    Font size of vertex annotations.
   * @param backColor   Background color.
   * @param blackCircle If true, draw black circle around each vertex.
   * @param scalePos    Scaling factor for the layout to adjust to drawing area.
   */
  public WriterDataGraphicsVRML(GraphData graph, 
                                PrintWriter out,
                                boolean hideSource, 
                                float minVert,
                                int fontSize,
                                Color backColor,
                                boolean blackCircle, 
                                float scalePos) {
	super(graph, hideSource, minVert, fontSize, backColor, blackCircle);
    this.out = out;
	this.scalePos = scalePos;
  }

  /*****************************************************************
   * Writes the layout in graphics format VRML.
   *****************************************************************/
  public void write() {
	int size = (int) (1000 * scalePos);

    // Header.
    out.print(
          "#VRML V2.0 utf8 " + endl
		+ "# " + endl 
	    + "# Generated by CCVisu, a tool for Co-Change Visualization " + endl 
	    + "#   and general force-directed graph layout. " + endl
	    + "#   " + DateFormat.getDateTimeInstance().format(new Date()) + " " + endl
	    + "  " + endl 
	    + "	PROTO TRANSFORM_Sphere [ " + endl
	    + "		field SFVec3f translation 0 0 0 " + endl
	    + "		field SFVec3f scale 1 1 1 " + endl
	    + "		field SFColor diffuseColor 0.5 0.5 0.5 " + endl
	    + "		field SFString name \"\" " + endl
	    + "		field SFVec3f textTranslate 0 0 1 " + endl
	    + "		field SFFloat textSize " + fontSize + endl
	    + "	] { " + endl
	    + "		Transform { " + endl
	    + "			translation IS translation " + endl
	    + "			children [ " + endl
	    + "				Transform { " + endl
	    + "					scale IS scale " + endl
	    + "					children [ " + endl
	    + "						DEF sensor0 TouchSensor {} " + endl
	    + "						Shape { " + endl
	    + "							geometry Sphere { radius 0.5 } " + endl
	    + "							appearance Appearance { " + endl
	    + "								material Material { " + endl
	    + "									ambientIntensity 0.3 " + endl
	    + "									diffuseColor IS diffuseColor " + endl
	    + "								} " + endl
	    + "							} " + endl
	    + "						} " + endl
	    + "					] " + endl
	    + "				} " + endl
	    + "				Billboard { " + endl
	    + "					axisOfRotation 0 0 0 " + endl
	    + "					children [ " + endl
	    + "						Transform { " + endl
	    + "							translation IS textTranslate " + endl
	    + "							children [ " + endl
	    + "								Shape { geometry DEF label0 Text { string [ \"\" ] fontStyle FontStyle { family \"SANS\" size IS textSize } } } " + endl
	    + "							] " + endl
	    + "						} " + endl
	    + "					] " + endl
	    + "				} " + endl
	    + "				DEF script0 Script { " + endl
	    + "					eventIn SFBool isOver " + endl
	    + "					eventIn SFBool touchTime " + endl
	    + "					eventOut MFString trans " + endl
	    + "					field SFBool labelDisplayed FALSE " + endl
	    + "					field SFString label IS name " + endl
	    + "					url [ \"javascript: " + endl
	    + "						function isOver(value) { " + endl
	    + "							if (value) { " + endl
	    + "								trans[0] = label; " + endl
	    + "							} else { " + endl
	    + "								if (!labelDisplayed) trans[0] = ''; " + endl
	    + "							} " + endl
	    + "						} " + endl
	    + "						function touchTime() { " + endl
	    + "							if (labelDisplayed) { " + endl
	    + "								trans[0] = ''; " + endl
	    + "								labelDisplayed = false; " + endl
	    + "							} else { " + endl
	    + "								trans[0] = label; " + endl
	    + "								labelDisplayed = true; " + endl
	    + "							} " + endl
	    + "						}\" " + endl
	    + "					] " + endl
    	+ "				} " + endl
	    + "			] " + endl
	    + "		} " + endl
	    + "		ROUTE sensor0.isOver TO script0.isOver " + endl
	    + "		ROUTE sensor0.touchTime TO script0.touchTime " + endl
	    + "		ROUTE script0.trans_changed TO label0.set_string " + endl
    	+ "	} " + endl + endl
    );

    // Body.
    writeGraphicsLayout(size);
  }

  /**
   * Writes a vertex in VRML format.
   * @param curVertex  The vertex object, to access vertex attributes.
   * @param xPos       x coordinate of the vertex.
   * @param yPos       y coordinate of the vertex.
   * @param zPos       z coordinate of the vertex.
   * @param dia        Diameter of the vertex.
   */
  public void writeVertex(GraphVertex curVertex, 
						  int xPos, int yPos, int zPos, int dia) {
	out.print(
          "TRANSFORM_Sphere { translation "
		+    xPos + " "
		+   -yPos + " "
		+    zPos + " "
		+ "scale " 
		+   dia + " "
		+   dia + " "
		+   dia + " " 
		+ "diffuseColor "
		+   curVertex.color.getRed() + " "
		+   curVertex.color.getGreen() + " "
		+   curVertex.color.getBlue() + " "
		+ "name "
		+   "\"" + curVertex.name + "\" textTranslate 0 0 " + (dia+1) + " } " + endl
	);
  }

};
